/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
/** @file
 *
 * @copyright &copy; Copyright 2007-2008 by Intra2net AG
 */

#include "source_track_basics.hpp"

#include <list>
#include <containerfunc.hpp>
#include <stringfunc.hxx>
#include <filefunc.hxx>


namespace I2n
{

/*
** implementation of SourceLocation
*/

SourceLocation::SourceLocation()
: Line ( 0 )
{
} // eo SourceLocation::SourceLocation()


SourceLocation::SourceLocation (
    const std::string& file,
    long line,
    const std::string& funcname
)
: File( file )
, Line( line )
, FunctionName( funcname )
{
} // eo SourceLocation::SourceLocation(const std::string&,long,const std::string&)


/**
 * @brief returns a "location tag"; i.e. a string which can be used to identify
 * the source location.
 * @return the location tag.
 */
std::string SourceLocation::get_location_tag() const
{
   static std::list< std::string > suffixes= TransientPushBackFiller< std::string, std::list >()
      ( ".cpp" ) ( ".cxx" ) ( ".cc" ) ( ".C" ) ( ".c++" )
      ( ".hpp" ) ( ".hxx" ) ( ".hh" ) ( ".H" ) ( ".h++" ) ( ".h" )
   ;
   if ( LocationTag.empty() and not File.empty() )
   {
      std::string str= basename( File );
      for( std::list< std::string >::const_iterator it= suffixes.begin();
           it != suffixes.end();
           ++it )
      {
         if ( has_suffix(str,*it) )
         {
            str= remove_suffix(str,*it);
            break;
         }
      }
      LocationTag= str;
      LocationTag+= "#";
      LocationTag+= to_string( Line );
   }
   return LocationTag;
} // eo SourceLocation::get_location_tag()



} // eo namespace I2n
